unit EditEvent;

interface

uses Classes, Controls, Forms, StdCtrls, ComCtrls, CalDAV;

type
  TEditEventForm = class(TForm)
    Label1: TLabel;
    WhatEd: TEdit;
    StartDateDTP: TDateTimePicker;
    StartTimeDTP: TDateTimePicker;
    EndDateDTP: TDateTimePicker;
    EndTimeDTP: TDateTimePicker;
    AllDayCB: TCheckBox;
    LocationEd: TEdit;
    DescriptMem: TMemo;
    OKBtn: TButton;
    GroupBox1: TGroupBox;
    RemindersLB: TListBox;
    RemAddBtn: TButton;
    RemEditBtn: TButton;
    RemDeleteBtn: TButton;
    procedure FormShow(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure AllDayCBClick(Sender: TObject);
    procedure RemindersLBClick(Sender: TObject);
    procedure RemAddBtnClick(Sender: TObject);
    procedure RemEditBtnClick(Sender: TObject);
    procedure RemDeleteBtnClick(Sender: TObject);
  protected
    procedure UpdateReminders;
    procedure UpdateReminderButtons;
  public
    Event: TCDEvent;
  end;

var
  EditEventForm: TEditEventForm;

implementation

uses SysUtils, EditRem;

{$R *.dfm}

const
  ReminderMethodStr: array[TReminderMethod] of string = ('Pop-up', 'Email');
  ReminderPeriodStr: array[TReminderPeriod] of string = ('minutes', 'hours', 'days', 'weeks');

procedure TEditEventForm.FormShow(Sender: TObject);
begin
  WhatEd.Text := Event.Title;
  LocationEd.Text := Event.Location;
  DescriptMem.Text := Event.Description;
  if Event.StartTime = 0 then
  begin
    StartDateDTP.DateTime := Date;
    StartTimeDTP.DateTime := Round(Now * 24) / 24;
    EndDateDTP.DateTime := Date;
    EndTimeDTP.DateTime := StartTimeDTP.DateTime + 1 / 24;
    AllDayCB.Checked := False;
  end
  else
  begin
    StartDateDTP.DateTime := Event.StartTime;
    StartTimeDTP.DateTime := Event.StartTime;
    EndDateDTP.DateTime := Event.EndTime;
    EndTimeDTP.DateTime := Event.EndTime;
    AllDayCB.Checked := Event.AllDay;
  end;
  UpdateReminders;
end;

procedure TEditEventForm.UpdateReminders;
var
  I: Integer;
  Reminder: TCDReminder;
begin
  RemindersLB.Clear;
  for I := 0 to Event.ReminderCount - 1 do
  begin
    Reminder := Event.GetReminder(I);
    RemindersLB.Items.Add(Format('%s %d %s', [ReminderMethodStr[Reminder.Method],
      Reminder.PeriodsCount, ReminderPeriodStr[Reminder.Period]]));
  end;
  UpdateReminderButtons;
end;

procedure TEditEventForm.UpdateReminderButtons;
begin
  RemEditBtn.Enabled := RemindersLB.ItemIndex >= 0;
  RemDeleteBtn.Enabled := RemindersLB.ItemIndex >= 0;
end;

procedure TEditEventForm.OKBtnClick(Sender: TObject);
begin
  Event.Title := WhatEd.Text;
  Event.AllDay := AllDayCB.Checked; // AllDay property must be set BEFORE StartTime and EndTime
  Event.StartTime := Trunc(StartDateDTP.DateTime) + Frac(StartTimeDTP.DateTime);
  Event.EndTime := Trunc(EndDateDTP.DateTime) + Frac(EndTimeDTP.DateTime);
  Event.Location := LocationEd.Text;
  Event.Description := DescriptMem.Text;
  Event.Store;
end;

procedure TEditEventForm.AllDayCBClick(Sender: TObject);
begin
  StartTimeDTP.Enabled := not AllDayCB.Checked;
  EndTimeDTP.Enabled := not AllDayCB.Checked;
  if AllDayCB.Checked then
  begin
    StartTimeDTP.DateTime := Date;
    EndTimeDTP.DateTime := Date;
  end
  else
  begin
    StartTimeDTP.DateTime := Round(Now * 24) / 24;
    EndTimeDTP.DateTime := StartTimeDTP.DateTime + 1 / 24;
  end
end;

procedure TEditEventForm.RemindersLBClick(Sender: TObject);
begin
  UpdateReminderButtons;
end;

procedure TEditEventForm.RemAddBtnClick(Sender: TObject);
var
  Reminder: TCDReminder;
begin
  ReminderForm.MethodCB.ItemIndex := 0;
  ReminderForm.PeriodCB.ItemIndex := 0;
  ReminderForm.NumberEd.Value := 5;
  if ReminderForm.ShowModal = mrOK then
    with Reminder do
    begin
      Method := TReminderMethod(ReminderForm.MethodCB.ItemIndex);
      Period := TReminderPeriod(ReminderForm.PeriodCB.ItemIndex);
      PeriodsCount := ReminderForm.NumberEd.Value;
      Event.AddReminder(Reminder);
      UpdateReminders;
    end;
end;

procedure TEditEventForm.RemEditBtnClick(Sender: TObject);
var
  Reminder: TCDReminder;
begin
  Reminder := Event.GetReminder(RemindersLB.ItemIndex);
  with Reminder do
  begin
    ReminderForm.MethodCB.ItemIndex := Integer(Method);
    ReminderForm.PeriodCB.ItemIndex := Integer(Period);
    ReminderForm.NumberEd.Value := PeriodsCount;
    if ReminderForm.ShowModal = mrOK then
    begin
      Method := TReminderMethod(ReminderForm.MethodCB.ItemIndex);
      Period := TReminderPeriod(ReminderForm.PeriodCB.ItemIndex);
      PeriodsCount := ReminderForm.NumberEd.Value;
      Event.SetReminder(RemindersLB.ItemIndex, Reminder);
      UpdateReminders;
    end;
  end;
end;

procedure TEditEventForm.RemDeleteBtnClick(Sender: TObject);
begin
  Event.DeleteReminder(RemindersLB.ItemIndex);
  RemindersLB.Items.Delete(RemindersLB.ItemIndex);
  UpdateReminderButtons;
end;

end.

